@@ -2,7 +2,8 @@
 
 void SendRemoteAreafix(void);
 struct Arealist *FindForward(uchar *tagname,uchar *flags);
-void RemoteAreafix(uchar *area,struct ConfigNode *node);
+void RemoteAreafixAdd(uchar *area,struct ConfigNode *node);
+void RemoteAreafixRemove(uchar *area,struct ConfigNode *node);
 bool CheckFlags(uchar group,uchar *node);
 
 struct afReply
@@ -113,9 +114,9 @@ bool AreaFix(struct MemMessage *mm)
 
    stop=FALSE;
 
-   for(chunk=(struct TextChunk *)mm->TextChunks.First;chunk && !stop;chunk=chunk->Next)
+   for(chunk=(struct TextChunk *)mm->TextChunks.First;chunk && !stop && !ctrlc;chunk=chunk->Next)
    {
-      for(c=0;c<chunk->Length && !stop;c++)
+      for(c=0;c<chunk->Length && !stop && !ctrlc;c++)
       {
          for(d=0;d<100 && chunk->Data[c+d]!=13 && chunk->Data[c+d]!=10 && c+d<chunk->Length;d++)
             buf[d]=chunk->Data[c+d];
@@ -356,7 +357,7 @@ bool AreaFix(struct MemMessage *mm)
                areaexists=FALSE;
                rescanarea=NULL;
 
-               for(area=(struct Area *)config.AreaList.First;area;area=area->Next)
+               for(area=(struct Area *)config.AreaList.First;area && !ctrlc;area=area->Next)
                   if(area->AreaType == AREATYPE_ECHOMAIL)
                   {
                      if(osMatchPattern(areaname,area->Tagname))
@@ -484,8 +485,10 @@ bool AreaFix(struct MemMessage *mm)
                                     wasfeed=FALSE;
 
                                     if(temptnode->Flags & TOSSNODE_FEED)
-                                       wasfeed=TRUE;												jbFreeNode(&area->TossNodes,(struct jbNode *)temptnode);
-												area->changed=TRUE;
+                                       wasfeed=TRUE;
+
+                                    jbFreeNode(&area->TossNodes,(struct jbNode *)temptnode);
+											   area->changed=TRUE;
                                     config.changed=TRUE;
 
                                     if(wasfeed && (config.cfg_Flags & CFG_REMOVEWHENFEED))
@@ -506,8 +509,7 @@ bool AreaFix(struct MemMessage *mm)
                                              if(area->TossNodes.First)
                                              {
                                                 LogWrite(3,AREAFIX,"AreaFix: Area %s removed, message sent to areafix",area->Tagname);
-                                                sprintf(buf,"-%s",area->Tagname);
-                                                RemoteAreafix(buf,((struct TossNode*)area->TossNodes.First)->ConfigNode);
+                                                RemoteAreafixRemove(area->Tagname,((struct TossNode*)area->TossNodes.First)->ConfigNode);
                                              }
                                              else
                                              {
@@ -575,16 +577,14 @@ bool AreaFix(struct MemMessage *mm)
 
 										RescanNode=cnode;
 								   	rescan_total=0;
-										isrescanning=TRUE;
 										success=(*rescanarea->Messagebase->rescanfunc)(rescanarea,rescannum,HandleRescan);
 										RescanNode=NULL;
-										isrescanning=FALSE;
-																				
+
 										if(!success)
 										{
 											afFreeReply(afr);
 	                              return(FALSE);
-										}									
+										}
 
                               LogWrite(4,AREAFIX,"AreaFix: Rescanned %lu messages",rescan_total);
                               afAddLine(afr,"%-30.30s Rescanned %lu messages","",rescan_total);
@@ -623,7 +623,7 @@ bool AreaFix(struct MemMessage *mm)
 
                                     sprintf(buf2,"Request sent to %u:%u/%u.%u",arealist->Node->Node.Zone,arealist->Node->Node.Net,arealist->Node->Node.Node,arealist->Node->Node.Point);
 												AddCommandReply(afr,buf,buf2);
-                                    RemoteAreafix(areaname,arealist->Node);
+                                    RemoteAreafixAdd(areaname,arealist->Node);
 
                                     area=AddArea(areaname,&arealist->Node->Node,&tmproute->Aka->Node,TRUE,config.cfg_Flags & CFG_FORWARDPASSTHRU);
                                     area->Group=arealist->Group;
@@ -696,7 +696,7 @@ bool AreaFix(struct MemMessage *mm)
    if(done==FALSE)
       afAddLine(afr,"Nothing to do.");
 
-   if(exitprg || nomem)
+   if(nomem || ioerror || ctrlc)
    {
       afFreeReply(afr);
       return(FALSE);
@@ -704,7 +704,7 @@ bool AreaFix(struct MemMessage *mm)
 
    afSendMessage(afr);
 	afFreeReply(afr);
-	
+
    if(sendarealist)
       rawSendList(SENDLIST_FULL,&tmproute->Aka->Node,mm->From,cnode);
 
@@ -760,9 +760,9 @@ void SendRemoveMessages(struct Area *area)
 
  			MakeFidoDate(time(NULL),mm->DateTime);  
  
-   		mm->isbounce=TRUE;
+   		mm->Flags |= MMFLAG_AUTOGEN;
 
-   		MakeNetmailKludges(mm);           
+   		MakeNetmailKludges(mm);
 
 		   if(config.cfg_Flags & CFG_ADDTID)
       		AddTID(mm);
@@ -773,7 +773,7 @@ void SendRemoveMessages(struct Area *area)
          sprintf(buf,"---\x0dGenerated by CrashMail "VERSION"\x0d");
 			mmAddLine(mm,buf);
 
-         HandleNetmail(mm);
+         HandleMessage(mm);
 			
 			mmFree(mm);
       }
@@ -800,9 +800,9 @@ void SendRemoveMessages(struct Area *area)
 
  			MakeFidoDate(time(NULL),mm->DateTime);  
  
-   		mm->isbounce=TRUE;
+   		mm->Flags |= MMFLAG_AUTOGEN;
 
-   		MakeNetmailKludges(mm);           
+   		MakeNetmailKludges(mm);
 
 		   if(config.cfg_Flags & CFG_ADDTID)
       		AddTID(mm);
@@ -816,7 +816,7 @@ void SendRemoveMessages(struct Area *area)
             mmAddLine(mm,buf);
          }
 
-         HandleNetmail(mm);
+         HandleMessage(mm);
 
          mmFree(mm);
       }
@@ -1316,7 +1316,7 @@ void rawSendHelp(struct Node4D *from4d,uchar *toname,struct ConfigNode *cnode)
    }
    else
    {
-      while(osFGets(fh,helpbuf,100) && !nomem && !exitprg)
+      while(osFGets(fh,helpbuf,100) && !nomem)
       {
          if(helpbuf[0]!=0)
             helpbuf[strlen(helpbuf)-1]=0;
@@ -1449,8 +1449,8 @@ struct afReply *afInitReply(uchar *fromname,struct Node4D *from4d,uchar *toname,
 	afr->mm->Attr = FLAG_PVT;
 
 	MakeFidoDate(time(NULL),afr->mm->DateTime);  
-	afr->mm->isbounce=TRUE;
-	MakeNetmailKludges(afr->mm);           
+	afr->mm->Flags |= MMFLAG_AUTOGEN;
+	MakeNetmailKludges(afr->mm);
 
    afr->lines=0;
    afr->part=1;
@@ -1478,8 +1478,8 @@ void afAddLine(struct afReply *afr,uchar *fmt,...)
       jbFreeList(&afr->mm->TextChunks);
 
 		MakeFidoDate(time(NULL),afr->mm->DateTime);  
-	 	afr->mm->isbounce=TRUE;
-		MakeNetmailKludges(afr->mm);           
+	 	afr->mm->Flags |= MMFLAG_AUTOGEN;
+		MakeNetmailKludges(afr->mm);
 
       strcpy(buf,"(Continued from previous message)\x0d\x0d");
       mmAddLine(afr->mm,buf);
@@ -1502,14 +1502,15 @@ void afSendMessage(struct afReply *afr)
 {
    if(afr->part != 1) 
 		sprintf(afr->mm->Subject,"%s (part %ld)",afr->subject,afr->part);
-		
+
 	else					
 		strcpy(afr->mm->Subject,afr->subject);
 		
-   HandleNetmail(afr->mm);
+   HandleMessage(afr->mm);
 }
 
-void RemoteAreafix(uchar *area,struct ConfigNode *node)
+
+void RemoteAreafixAdd(uchar *area,struct ConfigNode *node)
 {
    struct RemoteAFCommand *cmd;
 
@@ -1519,7 +1520,28 @@ void RemoteAreafix(uchar *area,struct ConfigNode *node)
       return;
    }
 
-   strncpy(cmd->Command,area,70);
+	if(node->Flags & NODE_AFNEEDSPLUS)
+	   sprintf(cmd->Command,"+%.78s",area);
+
+	else
+	   sprintf(cmd->Command,"%.78s",area);
+
+   cmd->Command[70]=0;
+   jbAddNode(&node->RemoteAFList,(struct jbNode *)cmd);
+}
+
+void RemoteAreafixRemove(uchar *area,struct ConfigNode *node)
+{
+   struct RemoteAFCommand *cmd;
+
+   if(!(cmd=(struct RemoteAFCommand *)osAllocCleared(sizeof(struct RemoteAFCommand))))
+   {
+      nomem=TRUE;
+      return;
+   }
+
+   sprintf(cmd->Command,"-%.78s",area);
+
    cmd->Command[70]=0;
    jbAddNode(&node->RemoteAFList,(struct jbNode *)cmd);
 }
@@ -1555,9 +1577,9 @@ void SendRemoteAreafix(void)
 
  			MakeFidoDate(time(NULL),mm->DateTime);  
  
-   		mm->isbounce=TRUE;
+   		mm->Flags |= MMFLAG_AUTOGEN;
 
-   		MakeNetmailKludges(mm);           
+   		MakeNetmailKludges(mm);
 
 		   if(config.cfg_Flags & CFG_ADDTID)
       		AddTID(mm);
@@ -1570,7 +1592,7 @@ void SendRemoteAreafix(void)
 
          mmAddLine(mm,"---\x0dGenerated by CrashMail "VERSION"\x0d");
 
-         HandleNetmail(mm);
+         HandleMessage(mm);
 
          mmFree(mm);
 
